@@ -17,6 +17,7 @@
 #include <time.h>
 
 #include "ipxlib.h"
+#include "com_err.h"
 
 #ifndef memzero
 #include <string.h>
@@ -83,41 +84,37 @@ struct ncp_search_seq {
    the argument list, opens the connection and removes the arguments
    from the list. It was designed after the X Windows init
    functions. */
-int
-ncp_initialize(struct ncp_conn *conn,
-	       int *argc, char **argv,
-	       int login_necessary);
+struct ncp_conn *
+ncp_initialize(int *argc, char **argv,
+	       int login_necessary, long *err);
 
 /* You can login as another object by this procedure. As a first use
    pserver comes to mind. */
-int
-ncp_initialize_as(struct ncp_conn *conn,
-		  int *argc, char **argv,
-		  int login_necessary, int login_type);
+struct ncp_conn *
+ncp_initialize_as(int *argc, char **argv,
+		  int login_necessary, int login_type, long *err);
 
 
 /* Open an existing permanent connection */
-int
-ncp_open(struct ncp_conn *conn,
-	 const struct ncp_conn_spec *spec);
+struct ncp_conn *
+ncp_open(const struct ncp_conn_spec *spec, long *err);
 
 /* Open a connection on an existing mount point */
-int
-ncp_open_mount(struct ncp_conn *conn,
-	       const char *mount_point);
+struct ncp_conn *
+ncp_open_mount(const char *mount_point, long *err);
 
-/* Detach from and destroy a permanent connection */
-int
-ncp_destroy_permanent(struct ncp_conn *conn);
+/* Find a permanent connection that fits the spec, return NULL if
+ * there is none. */
+char *
+ncp_find_permanent(const struct ncp_conn_spec *spec);
 
-/* Create a temporary connection */
-int
-ncp_open_temporary(struct ncp_conn *conn,
-		   const struct ncp_conn_spec *spec);
+/* Find the address of a file server */
+struct sockaddr_ipx *
+ncp_find_fileserver(const char *server_name, long *err);
 
 /* Detach from a permanent connection or destroy a temporary
    connection */
-int
+long
 ncp_close(struct ncp_conn *conn);
 
 /* like getmntent, get_ncp_conn_ent scans /etc/mtab for usable
@@ -140,124 +137,152 @@ ncp_get_conn_ent(FILE *filep);
 
 struct ncp_conn_spec *
 ncp_find_conn_spec(const char *server, const char *user, const char *password,
-		   uid_t uid);
+		   uid_t uid, long *err);
 
-int
+long
 ncp_get_file_server_description_strings(struct ncp_conn *conn,
 					char target[512]);
 
-int
+long
 ncp_get_file_server_time(struct ncp_conn *conn, time_t *target);
 
-int
-ncp_get_connlist(struct ncp_conn *conn, __u32 last_id,
+long
+ncp_get_connlist(struct ncp_conn *conn,
 		 __u16 object_type, const char *object_name,
-		 int *returned_no, __u16 conn_numbers[256]);
+		 int *returned_no, __u8 conn_numbers[256]);
 
-int
+long
+ncp_send_broadcast(struct ncp_conn *conn,
+		   __u8 no_conn, const __u8 *connections,
+		   const char *message);
+
+long
 ncp_get_encryption_key(struct ncp_conn *conn,
 		       char *target);
-int
+long
 ncp_get_bindery_object_id(struct ncp_conn *conn,
 			  __u16 object_type,
 			  const char *object_name,
 			  struct ncp_bindery_object *target);
-int
+long
 ncp_scan_bindery_object(struct ncp_conn *conn,
 			__u32 last_id, __u16 object_type, char *search_string,
 			struct ncp_bindery_object *target);
-int
+
+struct ncp_prop_login_control {
+    __u8    AccountExpireDate[3]     __attribute__ ((packed));
+    __u8    Disabled                 __attribute__ ((packed));
+    __u8    PasswordExpireDate[3]    __attribute__ ((packed));
+    __u8    GraceLogins              __attribute__ ((packed));
+    __u16   PasswordExpireInterval   __attribute__ ((packed));
+    __u8    MaxGraceLogins           __attribute__ ((packed));
+    __u8    MinPasswordLength        __attribute__ ((packed));
+    __u16   MaxConnections           __attribute__ ((packed));
+    __u8    ConnectionTimeMask[42]   __attribute__ ((packed));
+    __u8    LastLogin[6]             __attribute__ ((packed));
+    __u8    RestrictionMask          __attribute__ ((packed));
+    __u8    reserved                 __attribute__ ((packed));
+    __u32   MaxDiskUsage             __attribute__ ((packed));
+    __u16   BadLoginCount            __attribute__ ((packed));
+    __u32   BadLoginCountDown        __attribute__ ((packed));
+    __u8    LastIntruder[8]          __attribute__ ((packed));
+};	
+
+long
 ncp_read_property_value(struct ncp_conn *conn,
 			int object_type, const char *object_name,
 			int segment, const char *prop_name,
 			struct nw_property *target);
-int
+long
 ncp_login_encrypted(struct ncp_conn *conn,
 		    const struct ncp_bindery_object *object,
 		    const unsigned char *key,
 		    const unsigned char *passwd);
-int
+
+#define NCP_GRACE_PERIOD (0xdf)
+
+long
 ncp_login_user(struct ncp_conn *conn,
 	       const unsigned char *username,
 	       const unsigned char *password);
-int
+long
 ncp_get_volume_info_with_number(struct ncp_conn *conn, int n,
 				struct ncp_volume_info *target);
 
-int
+long
 ncp_get_volume_number(struct ncp_conn *conn, const char *name,
 		      int *target);
 
-int
+long
 ncp_file_search_init(struct ncp_conn *conn,
 		     int dir_handle, const char *path,
 		     struct ncp_filesearch_info *target);
 
-int
+long
 ncp_file_search_continue(struct ncp_conn *conn,
 			 struct ncp_filesearch_info *fsinfo,
 			 int attributes, const char *path,
 			 struct ncp_file_info *target);
 
-int
+long
 ncp_get_finfo(struct ncp_conn *conn,
 	      int dir_handle, const char *path, const char *name,
 	      struct ncp_file_info *target);
 
-int
+long
 ncp_open_file(struct ncp_conn *conn,
 	      int dir_handle, const char *path,
 	      int attr, int access,
 	      struct ncp_file_info *target);
-int
+long
 ncp_close_file(struct ncp_conn *conn, const char *file_id);
 
-int
+long
 ncp_create_newfile(struct ncp_conn *conn,
 		   int dir_handle, const char *path,
 		   int attr,
 		   struct ncp_file_info *target);
 
-int
+long
 ncp_create_file(struct ncp_conn *conn,
 		int dir_handle, const char *path,
 		int attr,
 		struct ncp_file_info *target);
 
-int
+long
 ncp_erase_file(struct ncp_conn *conn,
 	       int dir_handle, const char *path,
 	       int attr);
 
-int
+long
 ncp_rename_file(struct ncp_conn *conn,
 		int old_handle, const char *old_path,
 		int attr,
 		int new_handle, const char *new_path);
 
-int
+long
 ncp_create_directory(struct ncp_conn *conn,
 		     int dir_handle, const char *path,
 		     int inherit_mask);
 
-int
+long
 ncp_delete_directory(struct ncp_conn *conn,
 		     int dir_handle, const char *path);
 
-int
+long
 ncp_rename_directory(struct ncp_conn *conn,
 		     int dir_handle,
 		     const char *old_path, const char *new_path);
 
-int
+long
 ncp_read(struct ncp_conn *conn, const char *file_id,
 	 off_t offset, size_t count, char *target);
 
-int
+long
 ncp_write(struct ncp_conn *conn, const char *file_id,
 	  off_t offset, size_t count, const char *source);
 
-int
+long
 ncp_copy_file(struct ncp_conn *conn,
 	      const char source_file[6],
 	      const char target_file[6],
@@ -266,24 +291,24 @@ ncp_copy_file(struct ncp_conn *conn,
 	      __u32 count,
 	      __u32 *copied_count);
 
-int
+long
 ncp_do_lookup(struct ncp_conn *conn,
 	      struct nw_info_struct *dir,
 	      char *path,	/* may only be one component */
 	      struct nw_info_struct *target);
 
-int
+long
 ncp_modify_file_or_subdir_dos_info(struct ncp_conn *conn,
 				   struct nw_info_struct *file,
 				   __u32 info_mask,
 				   struct nw_modify_dos_info *info);
 
-int
+long
 ncp_del_file_or_subdir(struct ncp_conn *conn,
 		       struct nw_info_struct *dir, char *name);
 
 
-int
+long
 ncp_open_create_file_or_subdir(struct ncp_conn *conn,
 			       struct nw_info_struct *dir, char *name,
 			       int open_create_mode,
@@ -291,63 +316,63 @@ ncp_open_create_file_or_subdir(struct ncp_conn *conn,
 			       int desired_acc_rights,
 			       struct nw_file_info *target);
 
-int
+long
 ncp_initialize_search(struct ncp_conn *conn,
 		      const struct nw_info_struct *dir,
 		      int    namespace,
 		      struct ncp_search_seq *target);
 
-int
+long
 ncp_search_for_file_or_subdir(struct ncp_conn *conn,
 			      struct ncp_search_seq *seq,
 			      struct nw_info_struct *target);
 
-int
+long
 ncp_ren_or_mov_file_or_subdir(struct ncp_conn *conn,
 			      struct nw_info_struct *old_dir, char *old_name,
 			      struct nw_info_struct *new_dir, char *new_name);
 
-int
+long
 ncp_create_queue_job_and_file(struct ncp_conn *conn,
 			      __u32 queue_id,
 			      struct queue_job *job);
 
-int
+long
 ncp_close_file_and_start_job(struct ncp_conn *conn,
 			     __u32 queue_id,
 			     struct queue_job *job);
 
-int
+long
 ncp_attach_to_queue(struct ncp_conn *conn,
 		    __u32 queue_id);
 
-int
+long
 ncp_detach_from_queue(struct ncp_conn *conn,
 		      __u32 queue_id);
 
-int
+long
 ncp_service_queue_job(struct ncp_conn *conn, __u32 queue_id, __u16 job_type,
 		      struct queue_job *job);
 
-int
+long
 ncp_finish_servicing_job(struct ncp_conn *conn, __u32 queue_id,
 			 __u32 job_number, __u32 charge_info);
 
-int
+long
 ncp_abort_servicing_job(struct ncp_conn *conn, __u32 queue_id,
 			__u32 job_number);
 
-int
+long
 ncp_get_broadcast_message(struct ncp_conn *conn, char message[256]);
 
-int
+long
 ncp_dealloc_dir_handle(struct ncp_conn *conn, __u8 dir_handle);
 
 #define NCP_ALLOC_PERMANENT (0x0000)
 #define NCP_ALLOC_TEMPORARY (0x0001)
 #define NCP_ALLOC_SPECIAL   (0x0002)
 
-int
+long
 ncp_alloc_short_dir_handle(struct ncp_conn *conn,
 			   struct nw_info_struct *dir,
 			   __u16   alloc_mode,
