@@ -45,6 +45,7 @@ extern pid_t waitpid(pid_t, int *, int);
 #include <linux/ncp_fs.h>
 #include <linux/ncp_mount.h>
 #include "ncplib.h"
+#include "com_err.h"
 
 static char *progname;
 static void usage(void);
@@ -153,9 +154,11 @@ main(int argc, char *argv[])
 
         int fd, result;
 	struct sockaddr_ipx addr;
+	struct sockaddr_ipx *server_addr;
 	int addrlen;
 
         int upcase_password;
+	long err;
 
         int um;
 	unsigned int flags;
@@ -163,6 +166,7 @@ main(int argc, char *argv[])
         char mount_point[MAXPATHLEN];
         struct mntent ment;
         FILE *mtab;
+	char *tmp_mount;
 
         char *server   = NULL;
 	char *user     = NULL;
@@ -171,13 +175,13 @@ main(int argc, char *argv[])
 
 	uid_t conn_uid = getuid();
 
-	struct ncp_conn conn;
+	struct ncp_conn *conn;
 
 	int opt;
 
         progname = argv[0];
 
-	memzero(data); memzero(spec); memzero(conn);
+	memzero(data); memzero(spec);
 
 	if (geteuid() != 0)
 	{
@@ -192,10 +196,13 @@ main(int argc, char *argv[])
         data.file_mode = (S_IRWXU|S_IRWXG|S_IRWXO) & ~um;
         data.dir_mode  = 0;
 	data.flags    |= NCP_MOUNT_SOFT;
+	data.time_out = 60;
+	data.retry_count = 5;
 
 	upcase_password = 1;
 
-        while ((opt = getopt (argc, argv, "CS:U:c:u:g:f:d:P:nhvV:")) != EOF)
+        while ((opt = getopt (argc, argv, "CS:U:c:u:g:f:d:P:nhvV:t:r:"))
+	       != EOF)
 	{
                 switch (opt)
 		{
@@ -296,6 +303,12 @@ main(int argc, char *argv[])
                 case 'n':
 			password = "";
                         break;
+		case 't':
+			data.time_out = atoi(optarg);
+			break;
+		case 'r':
+			data.retry_count = atoi(optarg);
+			break;
 		case 'h':
 			help();
 			exit(1);
@@ -308,10 +321,10 @@ main(int argc, char *argv[])
                 }
         }
 
-	if ((spec = ncp_find_conn_spec(server, user, password, data.uid))
+	if ((spec = ncp_find_conn_spec(server, user, password, data.uid, &err))
 	    == NULL)
 	{
-		fprintf(stderr, "Could not find valid server/user\n");
+		com_err(progname, err, "in find_conn_spec");
 		exit(1);
 	}
 
@@ -367,32 +380,28 @@ main(int argc, char *argv[])
                         data.dir_mode |= S_IXOTH;
         }
 
-	if (ncp_open(&conn, spec) != 0)
+	if ((tmp_mount = ncp_find_permanent(spec)) != NULL)
 	{
-		fprintf(stderr, "could not connect to server %s: %s\n",
-			server, strerror(errno));
-		exit(1);
-	}
 
-	if (conn.is_connected == CONN_PERMANENT)
-	{
 		fprintf(stderr,
 			"You already have mounted server %s\nas user "
 			"%s\non mount point %s\n", spec->server, spec->user,
-			conn.mount_point);
-		ncp_close(&conn);
+			tmp_mount);
 		exit(1);
 	}
 
-	data.serv_addr = conn.i.addr;
-
-	ncp_close(&conn);
+	if ((server_addr = ncp_find_fileserver(spec->server, &err)) == NULL)
+	{
+		com_err("ncpmount", err, "when trying to find %s",
+			spec->server);
+		exit(1);
+	}
+	data.serv_addr = *server_addr;
 
 	data.ncp_fd  = socket(AF_IPX, SOCK_DGRAM, PF_IPX);
 	if (data.ncp_fd == -1)
 	{
-		fprintf(stderr, "could not open ncp socket: %s\n",
-			strerror(errno));
+		com_err("ncpmount", err, "opening ncp_socket");
 		exit(1);
 	}
 
@@ -463,10 +472,11 @@ main(int argc, char *argv[])
 
 	flags = MS_MGC_VAL;
 
-	data.time_out = 20;
-	data.retry_count = 2;
+	strcpy(mount_name, spec->server);
+	strcat(mount_name, "/");
+	strcat(mount_name, spec->user);
 
-	result = mount(NULL, mount_point, "ncpfs", flags, (char *)&data);
+	result = mount(mount_name, mount_point, "ncpfs", flags, (char *)&data);
 
 	if (result < 0)
 	{
@@ -474,20 +484,47 @@ main(int argc, char *argv[])
 		exit(1);
 	}
 
-	if (   (ncp_open_mount(&conn, mount_point) != 0)
-	    || (ncp_login_user(&conn, spec->user, spec->password) != 0)
-	    || (ioctl(conn.mount_fid, NCP_IOC_CONN_LOGGED_IN, NULL) != 0))
+	if ((conn = ncp_open_mount(mount_point, &err)) == NULL)
 	{
-		fprintf(stderr, "%s: login failed\n", strerror(errno));
-		ncp_close(&conn);
+		com_err("ncpmount", err, "attempt to open mount point");
 		umount(mount_point);
 		exit(1);
 	}
-	ncp_close(&conn);
 
-	strcpy(mount_name, spec->server);
-	strcat(mount_name, "/");
-	strcat(mount_name, spec->user);
+	if ((err = ncp_login_user(conn, spec->user, spec->password)) != 0)
+	{
+		struct nw_property p;
+		struct ncp_prop_login_control *l
+			= (struct ncp_prop_login_control *)&p;
+
+		if (conn->completion != NCP_GRACE_PERIOD)
+		{
+			com_err("ncpmount", err, "in login");
+			fprintf(stderr, "Login denied\n");
+			ncp_close(conn);
+			umount(mount_point);
+			exit(1);
+		}
+
+		fprintf(stderr, "Your password has expired\n");
+
+		if ((err = ncp_read_property_value(conn, NCP_BINDERY_USER,
+						   spec->user, 1,
+						   "LOGIN_CONTROL", &p)) == 0)
+		{
+			fprintf(stderr, "You have %d login attempts left\n",
+				l->GraceLogins);
+		}
+	}
+
+	if ((err = ioctl(conn->mount_fid, NCP_IOC_CONN_LOGGED_IN, NULL)) != 0)
+	{
+		com_err("ncpmount", err, "in logged_indication");
+		ncp_close(conn);
+		umount(mount_point);
+		exit(1);
+	}
+	ncp_close(conn);
 
         ment.mnt_fsname = mount_name;
         ment.mnt_dir = mount_point;
@@ -545,12 +582,16 @@ help(void)
         printf("\n"
 	       "-S server      Server name to be used\n"
                "-U username    Username sent to server\n"
+	       "-V volume      Volume to mount, for NFS re-export\n"
                "-u uid         uid the mounted files get\n"
                "-g gid         gid the mounted files get\n"
                "-f mode        permission the files get (octal notation)\n"
                "-d mode        permission the dirs get (octal notation)\n"
 	       "-c uid         uid to identify the connection to mount on\n"
 	       "               Only makes sense for root\n"
+	       "-t time_out    Waiting time (in 1/100s) to wait for\n"
+	       "               an answer from the server. Default: 60\n"
+	       "-r retry_count Number of retry attempts. Default: 5\n"
                "-C             Don't convert password to uppercase\n"
                "-P password    Use this password\n"
                "-n             Do not use any password\n"
